var RouteNotFound = require("../errors/RouteNotFound").RouteNotFound;
var ActionNotFound = require("../errors/ActionNotFound").ActionNotFound;
var NoValidTargetDefined = require("../errors/NoValidTargetDefined").NoValidTargetDefined;
var RouteParameterNotSpecified = require("../errors/RouteParameterNotSpecified").RouteParameterNotSpecified;

class RouteLoader {

    constructor(options) {
        this.baseUrl = location.origin;
        this.options = options
    }

    async find(route) {
        const routes = await this.loadRoutes();
        const uri = this.getRouteUri(routes, route);
        return this.createCallableUrl(uri, route)
    }

    async loadRoutes() {
        let routes;

        if (this.options.cache)
            routes = localStorage.getItem("rouxRoutes");

        if (!routes) {
            const jsonRoutesSource = window.location.origin + "/" + "loadRoutesForRoux";
            const response = await fetch(jsonRoutesSource);
            routes = await response.json();
            // todo cache routes if cache set to true
        }

        return routes;
    }


    getRouteUri(routes, route) {
        if (route.name) {
            if (routes.hasOwnProperty(route.name))
                return routes[route.name].uri;
            else throw new RouteNotFound(route.name);
        }

        if (route.action) {
            for (let rName in routes) {
                if (routes[rName].action === route.action)
                    return routes[rName].uri;
            }
        } else throw new ActionNotFound(route.action);

        throw new NoValidTargetDefined;
    };

    createCallableUrl(uri, route) {

        if (route.params.length)
            uri = this.replaceUriParameters(uri, route);

        return this.baseUrl + "/" + uri;
    };

    replaceUriParameters(uri, route) {
        const rawSegments = uri.split("/");

        const segments = rawSegments.map((segment) => {
            let parameter;
            if (parameter = segment.match(/^{.*}$/g)) {
                let value = route.params.shift();

                if (!value) throw new RouteParameterNotSpecified(
                    route.name || route.action,
                    parameter[0]
                );

                return value
            }
            return segment;
        });
        return segments.join("/");
    };

}

module.exports.RouteLoader = RouteLoader;
